/**********************************************************************
* 2013 DeepThoughtSystems(P)Ltd.
*
* FileName:        CANMateDll.h
*
*
* SOFTWARE LICENSE AGREEMENT:
* DeepThoughtSystems(P)Ltd ("DeepThoughts") retains all
* ownership and intellectual property rights in the code accompanying
* this message and in all derivatives hereto.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Version       Date      				Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* 1.0			28-Feb-2013   				Initial version
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
*
* DESCRIPTION:
* Header file containining the protos of exported functions and necessary
* data structures for CANMateDll API
*********************************************************************/

#ifndef _CANMATE_DLL_H_
#define _CANMATE_DLL_H_

#ifndef WIN32
#define __stdcall
typedef int HANDLE;
typedef void* LPVOID;
#endif

#define MAX_SIZE 15000

// Error codes
#define CANMATE_ERROR_SUCCESS 0
#define CANMATE_ERROR_CMD_FAILED 1
#define CANMATE_ERROR_TIMEOUT 2
#define CANMATE_ERROR_RCV_OVERFLOW 3
#define CANMATE_RCV_ERROR 4


/*********************************************************************
*
*                            Baud rate definitions
*
*********************************************************************/
#define BAUD_RATE33K	1
#define BAUD_RATE50K	2
#define BAUD_RATE80K	3
#define BAUD_RATE83K	4
#define BAUD_RATE100K	5
#define BAUD_RATE125K	6
#define BAUD_RATE200K	7
#define BAUD_RATE250K	8
#define BAUD_RATE500K	9
#define BAUD_RATE625K	10
#define BAUD_RATE800K	11
#define BAUD_RATE1000K	12

/*********************************************************************
*
*                            CAN MODE DEFINITIONS
*
*********************************************************************/

#define ECAN_NORMAL_MODE		1
#define ECAN_LOOPBACK_MODE		2
#define	ECAN_LISTENONLY_MODE	3

// Structure definition for CAN Message
typedef struct
{
    unsigned char bExtended;
    unsigned char chTmStmpH;
    unsigned char chTmStmpL;
    unsigned char EArbId1;
    unsigned char EArbId0;
    unsigned char SArbId1;
    unsigned char SArbId0;
    unsigned char DLC;
    unsigned char D0;
    unsigned char D1;
    unsigned char D2;
    unsigned char D3;
    unsigned char D4;
    unsigned char D5;
    unsigned char D6;
    unsigned char D7;
} CANMsg;

typedef struct
{
    unsigned char chErr;
    unsigned char chTxErrCnt;
    unsigned char chRxErrCnt;
} CANEvent;

// Error code is a bit mask value. definitios are given below

#define UART_ERROR				0x01 	// 0x0000 0001
#define	CAN_TXERROR				0x02 	// 0x0000 0010
#define CAN_BUS_OFF				0x04 	// 0x0000 0100
#define CAN_BUF_OVERFLOW		0x08 	// 0x0000 1000
#define CAN_TX_PASSIVE			0x10	// 0x0001 0000
#define CAN_RX_PASSIVE			0x20	// 0x0010 0000
#define CAN_ERR_WARNING			0x40	// 0x0100 0000
#define PC_BUF_OVERFLOW			0x80	// 0x1000 0000



#ifndef WIN32
extern /*"C"*/ HANDLE __stdcall OpenCANMate( LPVOID pDataCallBk/* = NULL*/, LPVOID pEventCallBk/* = NULL*/);
extern /*"C"*/ int __stdcall CloseCANMate(HANDLE hDev);

extern /*"C"*/ int __stdcall GetCurrentBaudRate(HANDLE hDev, int* pData);
extern /*"C"*/ int __stdcall SetCANBaudRate(HANDLE hDev,char chBaudRate);

extern /*"C"*/ int __stdcall GetCurrentMode(HANDLE hDev, int* pData);
extern /*"C"*/ int __stdcall SetNormalMode(HANDLE hDev);
extern /*"C"*/ int __stdcall SetLoopbackMode(HANDLE hDev);

extern /*"C"*/ int __stdcall StartReception(HANDLE hDev);
extern /*"C"*/ int __stdcall StopReception(HANDLE hDev);

extern /*"C"*/ int __stdcall WriteCANMessage(HANDLE hDev,CANMsg* pMsg);

extern /*"C"*/ int __stdcall GetFirmwareVersion(HANDLE hDev, int* pData);

extern /*"C"*/ double __stdcall GetCANTimeStamp(unsigned char chTmStmpH, unsigned char chTmStmpL);
#else
extern "C" HANDLE __stdcall OpenCANMate( LPVOID pDataCallBk = NULL, LPVOID pEventCallBk = NULL );
extern "C" int __stdcall CloseCANMate(HANDLE hDev);

extern "C" int __stdcall GetCurrentBaudRate(HANDLE hDev, int* pData);
extern "C" int __stdcall SetCANBaudRate(HANDLE hDev,char chBaudRate);

extern "C" int __stdcall GetCurrentMode(HANDLE hDev, int* pData);
extern "C" int __stdcall SetNormalMode(HANDLE hDev);
extern "C" int __stdcall SetLoopbackMode(HANDLE hDev);

extern "C" int __stdcall StartReception(HANDLE hDev);
extern "C" int __stdcall StopReception(HANDLE hDev);

extern "C" int __stdcall WriteCANMessage(HANDLE hDev,CANMsg* pMsg);

extern "C" int __stdcall GetFirmwareVersion(HANDLE hDev, int* pData);

extern "C" double __stdcall GetCANTimeStamp(unsigned char chTmStmpH, unsigned char chTmStmpL);

#endif

typedef int (*EVNT_CALLBACK)(CANEvent *pnEvnt);
typedef int (*DATA_CALLBACK)(CANMsg *pMsg, int *nNumMsgs);

// changing the calling conventions to match with .NET app

//typedef int (__stdcall *EVNT_CALLBACK) (CANEvent *pnEvnt);
//typedef int (__stdcall *DATA_CALLBACK) (CANMsg *pMsg, int *nNumMsgs);

#endif
